__version__ = "0.2.1"

__title__ = "pygohome"
__description__ = "A 100% personal route optimizer in a known environment based on experience."
__url__ = "https://github.com/eumiro/pygohome/"
__uri__ = __url__
__doc__ = __description__ + " <" + __uri__ + ">"

__author__ = "Miroslav Šedivý"
__email__ = "eumiro@gmail.com"

__license__ = "BSD-3"
__copyright__ = "Copyright (c) 2020 Miroslav Šedivý"
